import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { QueryHandler } from '@nestjs/cqrs';
import { ConfigurationMessagerPresenter } from '../../adapters/secondaries/configuration-messager.presenter';
import { ConfigurationMessager } from '../../adapters/secondaries/configuration.messager';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';
import { PropagateConfigurationsQuery } from '../../queries/propagate-configurations.query';
import { Configuration } from '../entities/configuration';

@QueryHandler(PropagateConfigurationsQuery)
export class PropagateConfigurationsUseCase {
  constructor(
    private readonly _repository: ConfigurationRepository,
    private readonly _configurationMessager: ConfigurationMessager,
    private readonly _loggingMessager: LoggingMessager,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  async execute(propagateConfigurationsQuery: PropagateConfigurationsQuery) {
    try {
      const configurations = await this._repository.findAll(1, 999999);
      this._configurationMessager.publish(
        'propagate',
        JSON.stringify(
          configurations.data.map((configuration) =>
            this._mapper.map(
              configuration,
              Configuration,
              ConfigurationMessagerPresenter,
            ),
          ),
        ),
      );
      this._loggingMessager.publish('configuration.update.info', 'propagation');
    } catch (error) {
      this._loggingMessager.publish('configuration.update.crit', 'propagation');
      throw error;
    }
  }
}
