import { Test, TestingModule } from '@nestjs/testing';
import { ICollection } from 'src/modules/database/src/interfaces/collection.interface';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { Domain } from '../../domain/dtos/domain.enum';
import { FindAllConfigurationsRequest } from '../../domain/dtos/find-all-configurations.request';
import { Configuration } from '../../domain/entities/configuration';
import { FindAllConfigurationsUseCase } from '../../domain/usecases/find-all-configurations.usecase';
import { FindAllConfigurationsQuery } from '../../queries/find-all-configurations.query';

const findAllConfigurationsRequest: FindAllConfigurationsRequest =
  new FindAllConfigurationsRequest();
findAllConfigurationsRequest.page = 1;
findAllConfigurationsRequest.perPage = 10;

const findAllConfigurationsQuery: FindAllConfigurationsQuery =
  new FindAllConfigurationsQuery(findAllConfigurationsRequest);

const mockConfigurations: ICollection<Configuration> = {
  data: [
    {
      uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
      domain: Domain.USER,
      key: 'key1',
      value: 'value1',
    },
    {
      uuid: 'bb281075-1b98-4456-89d6-c643d3044a92',
      domain: Domain.USER,
      key: 'key2',
      value: 'value2',
    },
    {
      uuid: 'bb281075-1b98-4456-89d6-c643d3044a93',
      domain: Domain.USER,
      key: 'key3',
      value: 'value3',
    },
  ],
  total: 3,
};

const mockConfigurationRepository = {
  findAll: jest
    .fn()
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    .mockImplementation((query?: FindAllConfigurationsQuery) => {
      return Promise.resolve(mockConfigurations);
    }),
};

describe('FindAllConfigurationsUseCase', () => {
  let findAllConfigurationsUseCase: FindAllConfigurationsUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: ConfigurationRepository,
          useValue: mockConfigurationRepository,
        },
        FindAllConfigurationsUseCase,
      ],
    }).compile();

    findAllConfigurationsUseCase = module.get<FindAllConfigurationsUseCase>(
      FindAllConfigurationsUseCase,
    );
  });

  it('should be defined', () => {
    expect(findAllConfigurationsUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should return an array filled with configurations', async () => {
      const configurations = await findAllConfigurationsUseCase.execute(
        findAllConfigurationsQuery,
      );

      expect(configurations).toBe(mockConfigurations);
    });
  });
});
